
#ifndef _IMAGE_H_
#define _IMAGE_H_

#include <global.h>

const char MAX_COLOR_PLANES = 3;
const char GREY  = 0;
const char RED   = 0;
const char GREEN = 1;
const char BLUE  = 2;

//--------------------------
// Vector class
//
template<class T> class CVector
{
  private:
    T* val;
    long length;
  public:
	CVector();
    CVector(long _len);
	CVector(long _start, long _end);
    T& operator[](int _i) { return val[_i]; }
	T*& getValPtr() { return val; }
    ~CVector() { delete[] val; }
};

//---------------------------
// Matrix Class
//
template<class T> class CMatrix
{
  private:
    CVector< CVector<T>* > *mat;
    long width;
    long height;
  public:
    CMatrix(long _width, long _height);
	CMatrix(long _colStart, long _colEnd, long _rowStart, long _rowEnd);
    T*& operator[](int i) { return (*mat)[i]->getValPtr(); }
    ~CMatrix();
};

//---------------------------
// Image Class
//
class CImage
{
  private:
    CMatrix<float>* band[ MAX_COLOR_PLANES ];   
    int symbols;                  // symbols after quantization
    int bpp;                      // bits per pixel (1, 8, 24)
    int colType;                  // (0 Color, 1 Grayscale)
    int colorPlanes;              // color planes - 3=rgb, 1=gray
    SaveFileFormat outFileType;   // output file format    
    char info[80];                // file info
  public:
    CImage(long _width, long _height, int _colorPlanes, int _bpp);
    ~CImage();
};

#endif //_IMAGE_H_ 


