
#ifndef __TRANSFORM_H__
#define __TRANSFORM_H__

#include "global.h"

typedef enum {
    FORWARD,      
    INVERSE        
} FLWTMode;

typedef struct {
    char infile[MAX_FILENAME],  // input data file 
         outfile[MAX_FILENAME], // output data file
         outFileType[MAX_FILETYPE]; // output file type 
    int sizeY,                  // height
        sizeX,                  // width
        N,                      // filter coefficients
        nTilde,                 // lifting coefficients
        gamma,                  // gamma correction 
        levels;                 // maximum levels for transform 
        FLWTMode mode;          // type of operation
        bool print,          // print coeffs?
        scale,                  // scale results? 
	    mallat;                 // Mallat format?
} ParamStruct;

extern void 
LiftingWaveletTranform ( ParamStruct __param );

extern void
GetDualWavelet ( const int __cols, const int __rows, const int __N, const int __nTilde,
            const bool __print_coeff, const int __levels );

extern void
FreeMemory ( const int __cols, const int __rows, const int __N, const int __nTilde,
            const int __levels );

extern void
GetMoments ( const int __cols, const int __rows, const int __N, const int __nTilde,
          const bool __print_coeffs, const int __levels );

extern void 
Lifting2DWT ( Matrix __data, const int __cols, const int __rows, const int __N,
            const int __nTilde, const int __levels, const bool __inverse );

extern void
Lifting1DWT ( Vector __data, const int __cols, const int __N,
                        const int __nTilde, const int __levels,
                        const bool __inverse );

extern void
HaarLifting2DWT ( Matrix __data, const int __cols, const int __rows,
                             const int __levels, const bool __inverse );

extern void
HaarLifting1DWT ( Vector __vect, const int __cols, const int __levels,
                             const bool __inverse );

extern void
Lifting1DWTPredict ( Vector __vect, const long __size, const long __incr,
                                const long __step, const long __N );

extern void
Lifting1DWTUpdate ( Vector __vect, const long __size, const long __incr,
                               const long __step, const long __nTilde,
                               const Vector __liftingCoeffs );

extern void
HaarLambdaCoeffs ( Vector __vect, const long __size, const long __incr,
                           const long __step );
                               
extern void
InvHaarLambdaCoeffs ( Vector __vect, const long __size, const long __incr,
                            const long __step );

extern void 
LiftingToMallat ( Matrix __Data, const int __width, const int __height,
                               const int __N, const int __nTilde,
                               const int __levels);

extern void 
MallatToLifting ( Matrix __Data, const int __width, const int __height,
                               const int __N, const int __nTilde,
                               const int __levels);

extern void 
LiftingToMallat1D ( Vector __Data, const int __len, const int __offset);

extern void 
MallatToLifting1D ( Vector __Data, const int __len, const int __offset);

#endif // __TRANSFORM_H__ 

