#include <stdlib.h>
#include <global.h>
#include <image.h>
#include <errors.h>

//---------------------------------------------
// CVector class member functions
//
inline template<class T> CVector<T>::CVector()
{
  length = 0;
  val = NULL;
}
inline template<class T> CVector<T>::CVector(int _len)
{
  if (_len <= 0) {
    Error("CVector Constructor : Invalid length argument", ABORT);
  }
  length = _len;
  val = new T[_len];
  if (!val) Error("CVector Constructor : Insufficient memory", ABORT);
}
inline template<class T> CVector<T>::CVector(int _start, int _end)
{
  length = _end - _start + 1;
  if (length <= 0) { 
    Error("CVector Constructor : Invalid arguments", ABORT);
  }
  val = new T[length];
  if (!val) Error("CVector Constructor : Insufficient memory", ABORT);
}
 
//---------------------------------------------
// CMatrix class member functions
//
inline template<class T> CMatrix<T>::CMatrix(long _width, long _height)
{
  width = _width;
  height = _height;
  if ((height <= 0) || (width <= 0)) {
    Error("CMatrix Constructor : Invalid dimensions", ABORT);
  } 
  mat = new CVec< CVec<T>* >(_height);
  if (!mat) Error("CMatrix Constructor : Insufficient memory", ABORT);
  for (int i=0; i < _height; i++) {
    (*mat)[i] = new CVec<T>(_width);
    if (!val) Error("CMatrix Constructor : Insufficient memory", ABORT);
  }
}
inline template<class T> CMatrix<T>::CMatrix(long _colStart, 
                      long _colEnd, long _rowStart, long _rowEnd);
{
  width = _colEnd - _colStart + 1 ;
  height = _rowEnd - _rowStart + 1;
  if ((height <= 0) || (width <= 0)) {
    Error("CMatrix Constructor : Invalid dimensions", ABORT);
  } 
  mat = new CVec< CVec<T>* >(_height);
  if (!mat) Error("CMatrix Constructor : Insufficient memory", ABORT);
  for (int i=0; i < _height; i++) {
    (*mat)[i] = new CVec<T>(_width);
    if (!(*mat)[i]) Error("CMatrix Constructor : Insufficient memory", ABORT);
  }
}
inline template<class T> CMatrix<T>::~CMatrix()
{
  for (int i=0; i < height; i++) {
    delete (*mat)[i];
  }
  delete mat;
}
//---------------------------------------------
// CImage class member functions
//
inline CImage::CImage(long _width, long _height, int _planes, int _bpp) 
{
  if (colorPlanes > MAX_COLOR_PLANES || colorPlanes <= 0) {
    Error("CImage Constructor : Invalid color planes value", ABORT);
  }
  if (width <= 0 || height <= 0) {
    Error("CImage Constructor : Invalid dimensions", ABORT);
  }
  width = _width;
  height = _height;
  colorPlanes = _planes;
  bpp = _bpp;
  for (int i=0; i < colorPlanes; i++) {
    band[i] = new CMatrix<float>(width, height);
  }
}
inline CImage::~CImage()
{
  for (int i=0; i < colorPlanes; i++) {
    delete band[i];
  }
}
