
#include <stdio.h>
#include <stdlib.h>
#include <global.h>     
#include <string.h>    
#include <errors.h>   
#include <transform.h>  
#include <util.h>     
//#include "windows.h"
#include <iostream.h>

static char *useStr[] = {
"Forward : main [-g|l|M|n|N|p|s] -t[outfile format] -o[outfile] [infile]",
"Inverse : main -i [-g|l|M|n|N|p|s] -t[outfile format] -o[outfile] [infile]", 
" ",
"-g <int>	sets value of gamma correction for scaling (default 1)",
"-i		execute the inverse transform (default FALSE)",
"-l <int>	sets maximum number of levels of the transform (default 999)",
"-M		input or output file is in Mallat format (default FALSE)", 
"-n <int>	sets number of real vanishing moments (defaul 4)",
"-N <int>	sets number of dual vanishing moments (default 2)",
"-o <outfile>	sets the output file name (default out.dat)",
"-p		prints intermediate results and coefficents",
"-s		executes the scaling of the output file",
"-t <type>	force type of output file (default same as input)",
"",
"infile		determines the input file (no default format)",
"",
"The available formats [-t] to read and write files are:",
"    pgm - portable graymap file format (raw and ascii)",
"    ppm - portable pixmap file format (raw and ascii)",
"    wgm - wavelet graymap file format",
"    wpm - wavelet pixmap file format",
"",
'\0'
};

void Usage();

//------------------------------------------------
// Main function
//------------------------------------------------
int main ( int argc, char **argv )
{
    static ParamStruct param;   /* used to store all the needed options */

    // Default values
    param.sizeX = param.sizeY = 0;
    param.levels = 999;
    param.gamma = 1;
    param.N = 2; param.nTilde = 4;
    param.mode = FORWARD;
    param.mallat = false;
    param.print = false;
    param.scale = false;
    strcpy (param.infile, "");
    sprintf (param.outfile, "out.dat");

    /* Parsing command line */
    while (--argc) {
        ++argv;
        if (*argv[0] == '-') {
            switch (argv[0][1]) {
                case 'g':   /* sets the value of the Gamma factor */
                    if ( argv[1] == NULL )
                        Error ("main", MISSING_PARAMETER, ABORT);
                    else
                        param.gamma = (int) atoi (argv[1]);
                    argc--; argv++;
                    break;
                case 'i':   /* apply Inverse transform */
                    param.mode = INVERSE;
                    break;
                case 'l':   /* maximum level of the transform */
                    param.levels = (int) atoi (argv[1]);
                    if (param.levels <= 0) {
                        Error ("main", INVALID_RELATIONAL_INT, ABORT, "J", ">", 0);
                    }
                    argc--; argv++;
                    break;
                case 'M':    /* set Mallat format to TRUE */
                    param.mallat = TRUE;
                    break;
                case 'n':   /* number of lifting coefficients */
                    if ( argv[1] == NULL )
                        Error ("main", MISSING_PARAMETER, ABORT);
                    else
                        param.nTilde = (int) atoi (argv[1]);
                    argc--; argv++;
                    break;
                case 'N':   /* number of filter coefficients */
                    if ( argv[1] == NULL )
                        Error ("main", MISSING_PARAMETER, ABORT);
                    else
                        param.N = (int) atoi (argv[1]);
                    argc--; argv++;
                    break;
                case 'o':   /* changes output file name */
                    if (argv[1] == NULL) {
                        Error ("main", MISSING_PARAMETER, ABORT);
                    } else {
                        sprintf (param.outfile, "%s", argv[1]);
                    }
                    argc--; argv++;
                    break;
                case 'p':   /* print coefficients */
                    param.print = TRUE;
                    break;
                case 's':   /* scales values between 0 and 255 */
                    param.scale = TRUE;
                    break;
                case 't':   /* set output file type */
                    if (argv[1] == NULL) {
                        Error ("main", MISSING_PARAMETER, ABORT);
                    } else {
                        strncpy (param.frmType, argv[1], MAX_FILETYPE-1);
                    }
                    argc--; argv++;
                    break;
                default:
                    Error ("main", IGNORE, RETURN, *argv);
                    break;
            }
        } else {
            sprintf (param.infile, "%s", *argv);
        }
    }

    /* Check for input file */
    if ( !strcmp (param.infile, "") ) {
        Usage ();
    }
    
    if ( ODD(param.N) ) {
        param.nTilde = 1;
    }
    if ( ODD(param.nTilde) ) {
        param.N = 1;
    }

    // Print Configuration 
    fprintf( stdout, "Input file name          [%s] \n", param.infile );
    fprintf( stdout, "Output file name         [%s] \n", param.outfile );
    if (param.scale) {
        fprintf (stdout, "Gamma correction        [%d] \n", param.gamma);
    }
    fprintf (stdout, "Print coefficients       [%s] \n", param.print ? "True" :
                                                                     "False");
    fprintf (stdout, "Scale values             [%s] \n", param.scale ? "True" :
                                                                    "False");
    fprintf (stdout, "Dual vanishing moments   [%d] \n", param.N);
    fprintf (stdout, "Real vanishing moments   [%d] \n", param.nTilde);
    fprintf (stdout, "Max num of levels        [%d] \n", param.levels);

    // Do Wavelet Transform
    LiftingWaveletTranform (param);

    // Finished
    //Sleep(100000);
    fprintf (stderr, "\nDone!\n");
    char c;
    cin>>c;
    return (SUCCESS); 
}

//------------------------------------------------
// Display help message 
//------------------------------------------------
static void Usage (void)
{
    char buf[BUFSIZ];
    int i;

    setbuf (stderr, buf);
    for ( i = 0 ; useStr[i] != NULL ; i++ )
        fprintf (stdout, "%s\n", useStr[i]);
    exit (SUCCESS);
}

