
#include "windows.h"

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <constant.h>
#include <global.h>
#include <util.h>

float
min3( float a, float b, float c )
{
    float m = MIN( a, b );
    return MIN( m, c );
}

float
max3( float a, float b, float c )
{
    float m = MAX( a, b );
    return MAX( m, c );
}

float
min4( float a, float b, float c, float d )
{
    float m = MIN( a, b );
    m = MIN( m, c );
    return MIN( m, d );
}

float
max4( float a, float b, float c, float d )
{
    float m = MAX( a, b );
    m = MAX( m, c );
    return MAX( m, d );
}

int
zeroEps( const float x, const float eps )
{
    return ( -eps < x ) && ( x < eps );
}

int
zero( const float x )
{
    const float __negeps_f = (float)-100 * (float) m_eps_f;
    const float __poseps_f = (float) 100 * (float) m_eps_f;

    return ( __negeps_f < x ) && ( x < __poseps_f );
}

float
logBaseN( const float x, const float n )
{
    return ( (float)((float)log((float)x)/(float)log((float)n)) );
}

/* should be called only for positive powers of two */
int 
logbasetwo( int x )
{
    int ret = 0;
  
    /* must be positive */
    assert( x > 0 );
    while( !(x & 1) ){
        x >>= 1;
        ret++;
    }
  
    /* must have been a power of two */
    assert( x == 1 );
    return ret;
}

/* true if x >=0 and x is a power of two */
int 
ispoweroftwo( int x )
{
    if( !x ) return 1;
    assert( x > 0 );
    while( !(x & 1) )
        x >>= 1;
    return x == 1;
}

/* given x, returns 2^ceil(log_2 x) */
int 
nextpoweroftwo( int x )
{
    int ret = 1;

    if( !x ) return 0;
    assert( x > 0 );
  
    while( ret < x )
        ret <<= 1;
    return ret;
}

class AbsRealTimer {
  double freq;
public:
  AbsRealTimer();
  ~AbsRealTimer();
  operator double() const;
  double operator ~() const { return double(*this); }
};

AbsRealTimer::AbsRealTimer() 
{
  LARGE_INTEGER f;
  int ok = QueryPerformanceFrequency(&f);
  freq = 1/double(f.QuadPart);
}

AbsRealTimer::~AbsRealTimer()
{
}

AbsRealTimer::operator double() const 
{ 
  LARGE_INTEGER time;
  QueryPerformanceCounter(&time);
  return double(time.QuadPart)*freq;
}

float get_current_time()
{
  return (float)AbsRealTimer();
}
